!================================================================
! file laf90mod.f90
!================================================================
! /netlib/lapack90   Preliminary version November, 1996
!
! file: laf90mod.f90
!       for: Interface module for the LAPACK Fortran 90 routines 
!================================================================


MODULE LAPACK90_INTERFACES
!
   INTERFACE LA_GESV
!
!  Purpose
!  =======
!
!  LA_GESV computes the solution to either a real or complex system of 
!  linear equations  AX = B,
!  where A is a square matrix and X and B are either rectangular
!  matrices or vectors.
!
!  The LU decomposition with partial pivoting and row interchanges is
!  used to factor A as  A = PLU,
!  where P is a permutation matrix, L is unit lower triangular, and U is
!  upper triangular. The factored form of A is then used to solve the
!  system of equations AX = B.
!
!  Arguments
!  =========
!
!  SUBROUTINE LA_GESV ( A, B, IPIV, INFO )
!     <type>(<wp>), INTENT(INOUT) :: A(:,:), <rhs>
!     INTEGER, INTENT(OUT), OPTIONAL :: IPIV(:)
!     INTEGER, INTENT(OUT), OPTIONAL :: INFO
!     where
!     <type> ::= REAL | COMPLEX
!     <wp>   ::= KIND(1.0) | KIND(1.0D0)
!     <rhs>  ::= B(:,:) | B(:)
!
!  =====================
!
!  A    (input/output) either REAL or COMPLEX square array,
!       shape (:,:), size(A,1) == size(A,2).
!       On entry, the matrix A.
!       On exit, the factors L and U from the factorization A = PLU;
!          the unit diagonal elements of L are not stored.
!
!  B    (input/output) either REAL or COMPLEX rectangular array,
!       shape either (:,:) or (:), size(B,1) or size(B) == size(A,1).
!       On entry, the right hand side vector(s) of matrix B for the
!          system of equations AX = B.
!       On exit, if there is no error, the matrix of solution
!          vector(s) X.
!
!  IPIV Optional (output) INTEGER array, shape (:),
!       size(IPIV) == size(A,1). If IPIV is present it indices that
!       define the permutation matrix P; row i of the matrix was
!       interchanged with row IPIV(i).
!
!  INFO Optional (output) INTEGER.
!       If INFO is present
!          = 0: successful exit
!          < 0: if INFO = -k, the k-th argument had an illegal value
!          > 0: if INFO = k, U(k,k) is exactly zero.  The factorization
!               has been completed, but the factor U is exactly
!               singular, so the solution could not be computed.
!       If INFO is not present and an error occurs, then the program is
!          terminated with an error message.
!
!  =====================================================================

      SUBROUTINE SGESV_F90(A,B,IPIV,INFO)
         USE LA_PRECISION, ONLY: SP
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(OUT), OPTIONAL :: IPIV(:)
         REAL(SP), INTENT(INOUT) :: A(:,:), B(:,:)
      END SUBROUTINE SGESV_F90
      SUBROUTINE S1GESV_F90(A,B,IPIV,INFO)
         USE LA_PRECISION, ONLY: SP
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(OUT), OPTIONAL :: IPIV(:)
         REAL(SP), INTENT(INOUT) :: A(:,:), B(:)
      END SUBROUTINE S1GESV_F90
      SUBROUTINE DGESV_F90(A,B,IPIV,INFO)
         USE LA_PRECISION, ONLY: DP
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(OUT), OPTIONAL :: IPIV(:)
         REAL(DP), INTENT(INOUT) :: A(:,:), B(:,:)
      END SUBROUTINE DGESV_F90
      SUBROUTINE D1GESV_F90(A,B,IPIV,INFO)
         USE LA_PRECISION, ONLY: DP
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(OUT), OPTIONAL :: IPIV(:)
         REAL(DP), INTENT(INOUT) :: A(:,:), B(:)
      END SUBROUTINE D1GESV_F90
      SUBROUTINE CGESV_F90(A,B,IPIV,INFO)
         USE LA_PRECISION, ONLY: SP
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(OUT), OPTIONAL :: IPIV(:)
         COMPLEX(SP), INTENT(INOUT) :: A(:,:), B(:,:)
      END SUBROUTINE CGESV_F90
      SUBROUTINE C1GESV_F90(A,B,IPIV,INFO)
         USE LA_PRECISION, ONLY: SP
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(OUT), OPTIONAL :: IPIV(:)
         COMPLEX(SP), INTENT(INOUT) :: A(:,:), B(:)
      END SUBROUTINE C1GESV_F90
      SUBROUTINE ZGESV_F90(A,B,IPIV,INFO)
         USE LA_PRECISION, ONLY: DP
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(OUT), OPTIONAL :: IPIV(:)
         COMPLEX(DP), INTENT(INOUT) :: A(:,:), B(:,:)
      END SUBROUTINE ZGESV_F90
      SUBROUTINE Z1GESV_F90(A,B,IPIV,INFO)
         USE LA_PRECISION, ONLY: DP
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(OUT), OPTIONAL :: IPIV(:)
         COMPLEX(DP), INTENT(INOUT) :: A(:,:), B(:)
      END SUBROUTINE Z1GESV_F90
   END INTERFACE
!
   INTERFACE LA_GESVX
!
!  Purpose
!  =======
!
!  LA_GESVX computes the solution to a either real or complex system of 
!  linear equations  AX = B,
!  where A is a square matrix and X and B are either rectangular
!  matrices or vectors.
!  LA_GESVX is an expert driver routine, which can also perform the
!  following functions:
!     - solve   A^T X = B  or  A^H X = B,
!     - estimate the condition number of A,
!     - return the pivot growth factor,
!     - refine the solution and computes forward and backward error
!       bounds,
!     - equilibrate the system if A is poorly scaled.
!
!  Arguments
!  =========
!  SUBROUTINE LA_GESVX (A, B, X, AF, IPIV, FACT, TRANS, EQUED, R, C, 
!                       FERR, BERR, RCOND, RPVGRW, INFO)
!     <type>(<wp>), INTENT(INOUT) :: A(:,:), <rsh>
!     <type>(<wp>), INTENT(OUT) :: <sol>
!     <type>(<wp>), INTENT(INOUT), OPTIONAL :: AF(:,:)
!     INTEGER, INTENT(INOUT), OPTIONAL :: IPIV(:)
!     CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
!     REAL(<wp>), INTENT(INOUT), OPTIONAL :: R(:), C(:)
!     REAL(<wp>), INTENT(OUT), OPTIONAL :: <err>, RCOND, RPVGRW
!     CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT, TRANS
!     INTEGER, INTENT(OUT), OPTIONAL :: INFO
!     where
!     <type> ::= REAL | COMPLEX
!     <wp>   ::= KIND(1.0) | KIND(1.0D0)
!     <rhs>  ::= B(:,:) | B(:)
!     <sol>  ::= X(:,:) | X(:)
!     <err>  ::= FERR(:), BERR(:) | FERR, BERR
!
!  Description
!  ===========
!
!  The following steps are performed:
!
!  1. If FACT is not present or FACT = 'N', and EQUED is present, 
!     real scaling factors are computed to equilibrate the system:
!        TRANS = 'N':  diag(R) A diag(C)    inv(diag(C)) X = diag(R) B
!        TRANS = 'T': (diag(R) A diag(C))^T inv(diag(R)) X = diag(C) B
!        TRANS = 'C': (diag(R) A diag(C))^H inv(diag(R)) X = diag(C) B
!     Whether or not the system will be equilibrated depends on the
!     scaling of the matrix A, but if equilibration is used, A is
!     overwritten by diag(R) A diag(C) and B by diag(R) B (if TRANS='N')
!     or diag(C) B (if TRANS = 'T' or 'C').
!
!  2. If FACT = 'N', the LU decomposition is used to factor the
!     matrix A (after equilibration if EQUED is present) as A = PLU,
!     where P is a permutation matrix, L is a unit lower triangular
!     matrix, and U is upper triangular.
!
!  3. The factored form of A is used to estimate the condition number
!     of the matrix A.  If the reciprocal of the condition number is
!     less than machine precision, steps 4-6 are skipped.
!
!  4. The system of equations is solved for X using the factored form
!     of A.
!
!  5. Iterative refinement is applied to improve the computed solution
!     matrix and calculate error bounds and backward error estimates
!     for it.
!
!  6. If equilibration was used, the matrix X is premultiplied by
!     diag(C) (if TRANS = 'N') or diag(R) (if TRANS = 'T' or 'C') so
!     that it solves the original system before equilibration.
!
!  Arguments
!  =========
!
!  A       (input/output) either REAL or COMPLEX square array,
!          shape (:,:), size(A,1) == size(A,2).
!          If FACT is not present or FACT = 'N',
!             On entry, the matrix A.
!             On exit, if EQUED is present, the matrix A may have been
!                overwritten by the equilibrated matrix (see EQUED).
!          If FACT is present and FACT = 'F',
!             On entry, the matrix A, possibly equilibrated in a previous
!                call to LA_GESVX (see EQUED).
!             On exit, A is unchanged.
!
!  B       (input/output) either REAL or COMPLEX rectangular array, 
!          shape either (:,:) or (:), size(B,1) or size(B) == size(A,1).
!          On entry, the right hand side vector(s) of matrix B for the
!             system of equations AX = B.
!          On exit, if EQUED is present, B may have been scaled in 
!             accordance with the equilibration of A (see EQUED); 
!             otherwise, B is unchanged.
!
!  X       (output) either REAL or COMPLEX rectangular array,
!          shape either (:,:) or (:), size(X,1) or size(X) == size(A,1).
!          If INFO = 0, the solution matrix X to the original
!          system of equations.  Note that X always returns the solution
!          to the original system of equations; if equilibration has been
!          performed (EQUED is present and EQUED /= 'N'), this does not
!          correspond to the scaled A and B.
!
!  AF      Optional (input/output) either REAL or COMPLEX square array,
!          shape (:,:), size(AF,1) == size(AF,2) == size(A,1).
!          If FACT is not present or FACT = 'N', then AF is an output 
!             argument and returns the factors L and U from the factorization 
!             A = PLU of the original matrix A, possibly equilibrated if
!             EQUED is present.
!          If FACT is present and FACT = 'F', then AF is an input argument 
!             (and must be present); on entry, it must contain the factors 
!             L and U of A (possibly equilibrated if EQUED is present), 
!             returned by a previous call to LA_GESVX.
!
!  IPIV    Optional (input/output) INTEGER array, shape (:), 
!          size(IPIV) == size(A,1).
!          If FACT is not present or FACT = 'N', then IPIV is an output 
!             argument and returns the pivot indices from the factorization 
!             A = PLU of the original matrix A, possibly equilibrated if
!             EQUED is present.
!          If FACT is present and FACT = 'F', then IPIV is an input argument 
!             (and must be present); on entry, it must contain the pivot 
!             indices from the factorization of A (possibly equilibrated 
!             if EQUED is present), returned by a previous call to LA_GESVX.
!
!  TRANS   Optional (input) CHARACTER*1
!          If TRANS is present, it specifies the form of the system 
!             of equations:
!             = 'N':  A X = B    (No transpose)
!             = 'T':  A^T X = B  (Transpose)
!             = 'C':  A^H X = B  (Conjugate transpose = Transpose)
!          otherwise TRANS = 'N' is assumed.
!
!  FACT    Optional (input) CHARACTER*1
!          Specifies whether or not the factored form of the matrix A is
!          supplied on entry.
!          If FACT is present then:
!             = 'N':  the matrix A will be equilibrated if EQUED is present,
!                then copied to AF and factored.
!             = 'F':  on entry, AF and IPIV must contain the factored form 
!                of A (possibly equilibrated if EQUED is present).
!          otherwise FACT = 'N' is assumed.
!
!  EQUED   Optional (input/output) CHARACTER*1
!          If FACT is not present or FACT = 'N', then EQUED is an output
!             argument. If it is present, then the matrix is equilibrated,
!             and on exit EQUED specifies the scaling of A which has 
!             actually been performed:
!             = 'N':  No equilibration.
!             = 'R':  Row equilibration, i.e., A has been premultiplied
!                     by diag(R); also B has been premultiplied by diag(R)
!                     if TRANS = 'N'.
!             = 'C':  Column equilibration, i.e., A has been postmultiplied 
!                     by diag(C); also B has been postmultiplied by diag(C)
!                     if TRANS = 'T' or 'C'.
!             = 'B':  Both row and column equilibration: combines the 
!                     effects of EQUED = 'R' and EQUED = 'C'.
!          If FACT is present and FACT = 'F', then EQUED is an input 
!             argument; if it is present, it specifies the equilibration
!             of A which was performed in a previous call to LA_GESVX with
!             FACT not present or FACT = 'N'.
!
!  R       Optional (input/output) REAL array, shape (:), 
!          size(R) == size(A,1).
!          R must be present if EQUED is present and EQUED = 'R' or 'B';
!          R is not referenced if EQUED = 'N' or 'C'.
!          If FACT is not present or FACT = 'N', then R is an output 
!             argument. If EQUED = 'R' or 'B', R returns the row scale 
!             factors for equilibrating A.
!          If FACT is present and FACT = 'F', then R is an input 
!             argument. If EQUED = 'R' or 'B', R must contain the row scale
!             factors for equilibrating A, returned by a previous call to 
!             LA_GESVX; each element of R must be positive.
!
!  C       Optional (input/output) REAL array, shape (:), 
!          size(C) == size(A,1).
!          C must be present if EQUED is present and EQUED = 'C' or 'B';
!          C is not referenced if EQUED = 'N' or 'R'.
!          If FACT is not present or FACT = 'N', then C is an output 
!             argument. If EQUED = 'C' or 'B', C returns the column scale 
!             factors for equilibrating A.
!          If FACT is present and FACT = 'F', then C is an input 
!             argument. If EQUED = 'C' or 'B', C must contain the column 
!             scale factors for equilibrating A, returned by a previous 
!             call to LA_GESVX; each element of C must be positive.
!
!  FERR    Optional (output) either REAL array of shape (:) or REAL
!          scalar. If it is an array, size(FERR) == size(X,2).
!          The estimated forward error bound for each solution vector
!          X(j) (the j-th column of the solution matrix X).
!          If XTRUE is the true solution corresponding to X(j), FERR(j)
!          is an estimated upper bound for the magnitude of the largest
!          element in (X(j) - XTRUE) divided by the magnitude of the
!          largest element in X(j).  The estimate is as reliable as
!          the estimate for RCOND, and is almost always a slight
!          overestimate of the true error.
!
!  BERR    Optional (output) either REAL array of shape (:) or REAL
!          scalar. If it is an array, size(BERR) == size(X,2).
!          The componentwise relative backward error of each solution
!          vector X(j) (i.e., the smallest relative change in
!          any element of A or B that makes X(j) an exact solution).
!
!  RCOND   Optional (output) REAL
!          The estimate of the reciprocal condition number of the matrix
!          A after equilibration (if done). If RCOND is less than the
!          machine precision (in particular, if RCOND = 0), the matrix
!          is singular to working precision. This condition is
!          indicated by a return code of INFO > 0, and the solution and
!          error bounds are not computed.
!
!  RPVGRW  Optional (output) REAL.
!          The reciprocal pivot growth factor norm(A)/norm(U).
!          If RPVGRW is much less than 1, then the stability
!          of the LU factorization of the (equilibrated) matrix A
!          could be poor. This also means that the solution X, condition
!          estimator RCOND, and forward error bound FERR could be
!          unreliable. If factorization fails with 0<INFO<=size(A,1),
!          then RPVGRW contains the reciprocal pivot growth factor for
!          the leading INFO columns of A.
!
!  INFO    Optional (output) INTEGER
!          If INFO is present
!             = 0:  successful exit
!             < 0:  if INFO = -k, the k-th argument had an illegal value
!             > 0:  if INFO = k, and k is
!                <= N:  U(k,k) is exactly zero.  The factorization has
!                       been completed, but the factor U is exactly
!                       singular, so the solution and error bounds
!                       could not be computed.
!                = N+1: RCOND is less than machine precision.  The
!                       factorization has been completed, but the
!                       matrix is singular to working precision, and
!                       the solution and error bounds have not been
!                       computed.
!          If INFO is not present and an error occurs, then the program is
!             terminated with an error message.
!
!  =====================================================================
!
      SUBROUTINE SGESVX_F90(A, B, X, AF, IPIV, FACT, TRANS, &
              EQUED, R, C, FERR, BERR, RCOND, RPVGRW, INFO)
         USE LA_PRECISION, ONLY: WP => SP
         CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS, FACT
         CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(INOUT), OPTIONAL :: IPIV(:)
         REAL(WP), INTENT(OUT), OPTIONAL :: RCOND, RPVGRW
         REAL(WP), INTENT(OUT), OPTIONAL :: FERR(:), BERR(:)
         REAL(WP), INTENT(INOUT), OPTIONAL :: AF(:,:), C(:), R(:)
         REAL(WP), INTENT(INOUT) :: A(:,:), B(:,:)
         REAL(WP), INTENT(OUT) :: X(:,:)
      END SUBROUTINE SGESVX_F90
      SUBROUTINE S1GESVX_F90(A, B, X, AF, IPIV, FACT, TRANS, &
              EQUED, R, C, FERR, BERR, RCOND, RPVGRW, INFO)
         USE LA_PRECISION, ONLY: WP => SP
         CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS, FACT
         CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(INOUT), OPTIONAL :: IPIV(:)
         REAL(WP), INTENT(OUT), OPTIONAL :: RCOND, FERR, BERR, RPVGRW
         REAL(WP), INTENT(INOUT), OPTIONAL :: AF(:,:), C(:), R(:)
         REAL(WP), INTENT(INOUT) :: A(:,:), B(:)
         REAL(WP), INTENT(OUT) :: X(:)
      END SUBROUTINE S1GESVX_F90
      SUBROUTINE DGESVX_F90(A, B, X, AF, IPIV, FACT, TRANS, &
              EQUED, R, C, FERR, BERR, RCOND, RPVGRW, INFO)
         USE LA_PRECISION, ONLY: WP => DP
         CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS, FACT
         CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(INOUT), OPTIONAL :: IPIV(:)
         REAL(WP), INTENT(OUT), OPTIONAL :: RCOND, RPVGRW
         REAL(WP), INTENT(OUT), OPTIONAL :: FERR(:), BERR(:)
         REAL(WP), INTENT(INOUT), OPTIONAL :: AF(:,:), C(:), R(:)
         REAL(WP), INTENT(INOUT) :: A(:,:), B(:,:)
         REAL(WP), INTENT(OUT) :: X(:,:)
      END SUBROUTINE DGESVX_F90
      SUBROUTINE D1GESVX_F90(A, B, X, AF, IPIV, FACT, TRANS, &
              EQUED, R, C, FERR, BERR, RCOND, RPVGRW, INFO)
         USE LA_PRECISION, ONLY: WP => DP
         CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS, FACT
         CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(INOUT), OPTIONAL :: IPIV(:)
         REAL(WP), INTENT(OUT), OPTIONAL :: RCOND, FERR, BERR, RPVGRW
         REAL(WP), INTENT(INOUT), OPTIONAL :: AF(:,:), C(:), R(:)
         REAL(WP), INTENT(INOUT) :: A(:,:), B(:)
         REAL(WP), INTENT(OUT) :: X(:)
      END SUBROUTINE D1GESVX_F90
      SUBROUTINE CGESVX_F90(A, B, X, AF, IPIV, FACT, TRANS, &
              EQUED, R, C, FERR, BERR, RCOND, RPVGRW, INFO)
         USE LA_PRECISION, ONLY: WP => SP
         CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS, FACT
         CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(INOUT), OPTIONAL :: IPIV(:)
         REAL(WP), INTENT(OUT), OPTIONAL :: RCOND, RPVGRW
         REAL(WP), INTENT(OUT), OPTIONAL :: FERR(:), BERR(:)
         REAL(WP), INTENT(INOUT), OPTIONAL :: C(:), R(:)
         COMPLEX(WP), INTENT(INOUT), OPTIONAL :: AF(:,:)
         COMPLEX(WP), INTENT(INOUT) :: A(:,:), B(:,:)
         COMPLEX(WP), INTENT(OUT) :: X(:,:)
      END SUBROUTINE CGESVX_F90
      SUBROUTINE C1GESVX_F90(A, B, X, AF, IPIV, FACT, TRANS, &
              EQUED, R, C, FERR, BERR, RCOND, RPVGRW, INFO)
         USE LA_PRECISION, ONLY: WP => SP
         CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS, FACT
         CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(INOUT), OPTIONAL :: IPIV(:)
         REAL(WP), INTENT(OUT), OPTIONAL :: RCOND, FERR, BERR, RPVGRW
         REAL(WP), INTENT(INOUT), OPTIONAL :: C(:), R(:)
         COMPLEX(WP), INTENT(INOUT), OPTIONAL :: AF(:,:)
         COMPLEX(WP), INTENT(INOUT) :: A(:,:), B(:)
         COMPLEX(WP), INTENT(OUT) :: X(:)
      END SUBROUTINE C1GESVX_F90
      SUBROUTINE ZGESVX_F90(A, B, X, AF, IPIV, FACT, TRANS, &
              EQUED, R, C, FERR, BERR, RCOND, RPVGRW, INFO)
         USE LA_PRECISION, ONLY: WP => DP
         CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS, FACT
         CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(INOUT), OPTIONAL :: IPIV(:)
         REAL(WP), INTENT(OUT), OPTIONAL :: RCOND, RPVGRW
         REAL(WP), INTENT(OUT), OPTIONAL :: FERR(:), BERR(:)
         REAL(WP), INTENT(INOUT), OPTIONAL :: C(:), R(:)
         COMPLEX(WP), INTENT(INOUT), OPTIONAL :: AF(:,:)
         COMPLEX(WP), INTENT(INOUT) :: A(:,:), B(:,:)
         COMPLEX(WP), INTENT(OUT) :: X(:,:)
      END SUBROUTINE ZGESVX_F90
      SUBROUTINE Z1GESVX_F90(A, B, X, AF, IPIV, FACT, TRANS, &
              EQUED, R, C, FERR, BERR, RCOND, RPVGRW, INFO)
         USE LA_PRECISION, ONLY: WP => DP
         CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS, FACT
         CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(INOUT), OPTIONAL :: IPIV(:)
         REAL(WP), INTENT(OUT), OPTIONAL :: RCOND, FERR, BERR, RPVGRW
         REAL(WP), INTENT(INOUT), OPTIONAL :: C(:), R(:)
         COMPLEX(WP), INTENT(INOUT), OPTIONAL :: AF(:,:)
         COMPLEX(WP), INTENT(INOUT) :: A(:,:), B(:)
         COMPLEX(WP), INTENT(OUT) :: X(:)
      END SUBROUTINE Z1GESVX_F90
   END INTERFACE
!
   INTERFACE LA_GETRF
!
!  Purpose
!  =======
!
!  LA_GETRF computes an LU factorization of a general regtangle
!  matrix A using partial pivoting with row interchanges.
!
!  The factorization has the form A = PLU
!  where P is a permutation matrix, L is lower triangular
!  with unit diagonal elements (lower trapezoidal if m > n),
!  and U is upper triangular (upper trapezoidal if m < n),
!  where m=size(A,1) and n=size(A,2).
! 
!  When A is square (m = n), LA_GETRF optionally estimates the 
!  reciprocal of the condition number of a general matrix A, in either 
!  the 1-norm or the infinity-norm.
!  An estimate is obtained for norm(inv(A)), and the reciprocal of the
!  condition number is computed as RCOND = 1 / (norm(A) * norm(inv(A))).
!
!  Arguments
!  =========
!
!  SUBROUTINE LA_GETRF ( A, IPIV, RCOND, NORM, INFO )
!     <type>(<wp>), INTENT(INOUT) :: A(:,:)
!     INTEGER, INTENT( OUT ) :: IPIV( : )
!     REAL(<wp>), INTENT( OUT ), OPTIONAL :: RCOND
!     CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
!     INTEGER, INTENT(OUT), OPTIONAL :: INFO
!     where
!     <type> ::= REAL | COMPLEX
!     <wp>   ::= KIND(1.0) | KIND(1.0D0)
!
!  ====================
!
!  A     (input/output) either REAL or COMPLEX array, shape (:,:).
!        On entry, the matrix A.
!        On exit, the factors L and U from the factorization A = PLU; 
!           the unit diagonal elements of L are not stored.
!
!  IPIV  (output) INTEGER array, shape (:),
!        size(IPIV) == min(size(A,1),size(A,2)). 
!        IPIV indices that define the permutation matrix P;
!        row i of the matrix A was interchanged with row IPIV(i).
!
!  RCOND Optional (output) REAL
!        The reciprocal of the condition number of the matrix A for
!        the case m = n, computed as RCOND = 1/(norm(A) * norm(inv(A))).
!        RCOND should be present if NORM is present. 
!        If m /= n then RCOND is returned as zero.
!
!  NORM  Optional (input) CHARACTER*1
!        Specifies whether the 1-norm condition number or the
!        infinity-norm condition number is required:
!           = '1', 'O' or 'o':  1-norm;
!           = 'I', 'i':         infinity-norm.
!        If NORM is not present, the 1-norm is used.
!
!  INFO  Optional (output) INTEGER.
!        If INFO is present
!           = 0: successful exit
!           < 0: if INFO = -k, the k-th argument had an illegal value
!           > 0: if INFO = k, U(k,k) is exactly zero.  The factorization
!                has been completed, but the factor U is exactly
!                singular, so the solution could not be computed.
!        If INFO is not present and an error occurs, then the program is
!           terminated with an error message.
!
!  =====================================================================
!
      SUBROUTINE SGETRF_F90( A,IPIV,RCOND,NORM,INFO )
         USE LA_PRECISION, ONLY: WP => SP
         CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT( OUT ) :: IPIV( : )
         REAL(WP), INTENT( INOUT ) :: A( :, : )
         REAL(WP), INTENT( OUT ), OPTIONAL :: RCOND
      END SUBROUTINE SGETRF_F90
      SUBROUTINE DGETRF_F90( A,IPIV,RCOND,NORM,INFO )
         USE LA_PRECISION, ONLY: WP => DP
         CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT( OUT ) :: IPIV( : )
         REAL(WP), INTENT( INOUT ) :: A( :, : )
         REAL(WP), INTENT( OUT ), OPTIONAL :: RCOND
      END SUBROUTINE DGETRF_F90
      SUBROUTINE CGETRF_F90( A,IPIV,RCOND,NORM,INFO )
         USE LA_PRECISION, ONLY: WP => SP
         CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT( OUT ) :: IPIV( : )
         COMPLEX(WP), INTENT( INOUT ) :: A( :, : )
         REAL(WP), INTENT( OUT ), OPTIONAL :: RCOND
      END SUBROUTINE CGETRF_F90
      SUBROUTINE ZGETRF_F90( A,IPIV,RCOND,NORM,INFO )
         USE LA_PRECISION, ONLY: WP => DP
         CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT( OUT ) :: IPIV( : )
         COMPLEX(WP), INTENT( INOUT ) :: A( :, : )
         REAL(WP), INTENT( OUT ), OPTIONAL :: RCOND
      END SUBROUTINE ZGETRF_F90
   END INTERFACE
!
   INTERFACE LA_GETRS
!
!  Purpose
!  =======
!
!  LA_GETRS solves a system of linear equations
!     A X = B, A^T X = B or  A^H X = B
!  with a general square matrix A using the LU factorization computed
!  by LA_GETRF.
!
!  Arguments
!  =========
!  SUBROUTINE LA_GETRS (A, IPIV, B, TRANS, INFO)
!     <type>(<wp>), INTENT(IN)  :: A(:,:)
!     <type>(<wp>), INTENT(INOUT) :: <rhs>
!     INTEGER, INTENT(IN) :: IPIV(:)
!     CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
!     INTEGER, INTENT(OUT), OPTIONAL :: INFO
!     where
!     <type> ::= REAL | COMPLEX
!     <wp>   ::= KIND(1.0) | KIND(1.0D0)
!     <rhs>  ::= B(:,:) | B(:)
!
!  =====================
!
!  A     (input) either REAL or COMPLEX square array,
!        shape (:,:), size(A,1) == size(A,2).
!        The factors L and U from the factorization A = PLU as computed 
!        by LA_GETRF.
!
!  IPIV  (input) INTEGER array, shape (:), size(IPIV) == size(A,1).
!        The pivot indices from LA_GETRF; for 1<=i<=size(A,1), row i
!        of the matrix was interchanged with row IPIV(i).
!
!  B     (input/output) either REAL or COMPLEX rectangular array,
!        shape either (:,:) or (:), size(B,1) or size(B) == size(A,1).
!        On entry, the right hand side vector(s) of matrix B for
!           the system of equations AX = B.
!        On exit, if there is no error, the matrix of solution
!           vector(s) X.
!
!  TRANS Optional (input) CHARACTER*1
!        If TRANS is present, it specifies the form of the system 
!           of equations:
!           = 'N':  A X = B    (No transpose)
!           = 'T':  A^T X = B  (Transpose)
!           = 'C':  A^H X = B  (Conjugate transpose = Transpose)
!        otherwise TRANS = 'N' is assumed.
!
!  INFO  Optional (output) INTEGER.
!        If INFO is present
!           = 0: successful exit
!           < 0: if INFO = -k, the k-th argument had an illegal value
!        If INFO is not present and an error occurs, then the program is
!           terminated with an error message.
!
!  =====================================================================
!
      SUBROUTINE SGETRS_F90(A,IPIV,B,TRANS,INFO)
         USE LA_PRECISION, ONLY: SP
         CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(IN) :: IPIV(:)
         REAL(SP), INTENT(IN) :: A(:,:)
         REAL(SP), INTENT(INOUT) :: B(:,:)
      END SUBROUTINE SGETRS_F90
      SUBROUTINE DGETRS_F90(A,IPIV,B,TRANS,INFO)
         USE LA_PRECISION, ONLY: DP
         CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(IN) :: IPIV(:)
         REAL(DP), INTENT(IN) :: A(:,:)
         REAL(DP), INTENT(INOUT) :: B(:,:)
      END SUBROUTINE DGETRS_F90
      SUBROUTINE CGETRS_F90(A,IPIV,B,TRANS,INFO)
         USE LA_PRECISION, ONLY: SP
         CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(IN) :: IPIV(:)
         COMPLEX(SP), INTENT(IN) :: A(:,:)
         COMPLEX(SP), INTENT(INOUT) :: B(:,:)
      END SUBROUTINE CGETRS_F90
      SUBROUTINE ZGETRS_F90(A,IPIV,B,TRANS,INFO)
         USE LA_PRECISION, ONLY: DP
         CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(IN) :: IPIV(:)
         COMPLEX(DP), INTENT(IN) :: A(:,:)
         COMPLEX(DP), INTENT(INOUT) :: B(:,:)
      END SUBROUTINE ZGETRS_F90
      SUBROUTINE S1GETRS_F90(A,IPIV,B,TRANS,INFO)
         USE LA_PRECISION, ONLY: SP
         CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(IN) :: IPIV(:)
         REAL(SP), INTENT(IN) :: A(:,:)
         REAL(SP), INTENT(INOUT) :: B(:)
      END SUBROUTINE S1GETRS_F90
      SUBROUTINE D1GETRS_F90(A,IPIV,B,TRANS,INFO)
         USE LA_PRECISION, ONLY: DP
         CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(IN) :: IPIV(:)
         REAL(DP), INTENT(IN) :: A(:,:)
         REAL(DP), INTENT(INOUT) :: B(:)
      END SUBROUTINE D1GETRS_F90
      SUBROUTINE C1GETRS_F90(A,IPIV,B,TRANS,INFO)
         USE LA_PRECISION, ONLY: SP
         CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(IN) :: IPIV(:)
         COMPLEX(SP), INTENT(IN) :: A(:,:)
         COMPLEX(SP), INTENT(INOUT) :: B(:)
      END SUBROUTINE C1GETRS_F90
      SUBROUTINE Z1GETRS_F90(A,IPIV,B,TRANS,INFO)
         USE LA_PRECISION, ONLY: DP
         CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(IN) :: IPIV(:)
         COMPLEX(DP), INTENT(IN) :: A(:,:)
         COMPLEX(DP), INTENT(INOUT) :: B(:)
      END SUBROUTINE Z1GETRS_F90
   END INTERFACE
!
   INTERFACE LA_GETRI
!
!  Purpose
!  =======
!
!  LA_GETRI computes the inverse of a matrix using the LU factorization 
!  computed by LA_GETRF.
!
!  Arguments
!  =========
!  SUBROUTINE LA_GETRI (A, IPIV, INFO)
!     <type>(<wp>), INTENT(INOUT)  :: A(:,:)
!     INTEGER, INTENT(IN) :: IPIV(:)
!     INTEGER, INTENT(OUT), OPTIONAL :: INFO
!     where
!     <type> ::= REAL | COMPLEX
!     <wp>   ::= KIND(1.0) | KIND(1.0D0)
!
!  =====================
!
!  A      (input/output) either REAL or COMPLEX square array, shape (:,:),
!         size(A,1) == size(A,2).
!         On entry contains the factors L and U from the factorization
!            A = PLU as computed by LA_GETRF.
!         On exit, if INFO = 0, the inverse of the original matrix A.
!
!  IPIV   (input) INTEGER array, shape (:), size(IPIV) == size(A,1).
!         The pivot indices from LA_GETRF; for 1<=i<=size(A,1), row i of
!         the matrix was interchanged with row IPIV(i).
!
!  INFO   Optional (output) INTEGER.
!         If INFO is present
!            = 0: successful exit
!            < 0: if INFO = -k, the k-th argument had an illegal value
!            > 0: if INFO = k, U(k,k) is exactly zero.  The matrix is
!                singular and its inverse could not be computed.
!         If INFO is not present and an error occurs, then the program is
!            terminated with an error message.
!
!  =====================================================================
!
      SUBROUTINE SGETRI_F90(A,IPIV,INFO)
         USE LA_PRECISION, ONLY: WP => SP
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(IN) :: IPIV(:)
         REAL(WP), INTENT(INOUT) :: A(:,:)
      END SUBROUTINE SGETRI_F90
      SUBROUTINE DGETRI_F90(A,IPIV,INFO)
         USE LA_PRECISION, ONLY: WP => DP
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(IN) :: IPIV(:)
         REAL(WP), INTENT(INOUT) :: A(:,:)
      END SUBROUTINE DGETRI_F90
      SUBROUTINE CGETRI_F90(A,IPIV,INFO)
         USE LA_PRECISION, ONLY: WP => SP
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(IN) :: IPIV(:)
         COMPLEX(WP), INTENT(INOUT) :: A(:,:)
      END SUBROUTINE CGETRI_F90
      SUBROUTINE ZGETRI_F90(A,IPIV,INFO)
         USE LA_PRECISION, ONLY: WP => DP
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(IN) :: IPIV(:)
         COMPLEX(WP), INTENT(INOUT) :: A(:,:)
      END SUBROUTINE ZGETRI_F90
   END INTERFACE
!
   INTERFACE LA_GERFS
!
!  Purpose
!  =======
!
!  LA_GERFS improves the computed solution X of a system of linear 
!  equations   A X = B  or  A^T X = B
!  and provides error bounds and backward error estimates for
!  the solution. LA_GERFS uses the LU factors computed by LA_GETRF.
!
!  Arguments
!  =========
!  SUBROUTINE LA_GERFS (A, AF, IPIV, B, X, TRANS, FERR, BERR, INFO)
!     <type>(<wp>), INTENT(IN)  :: A(:,:), AF(:,:), <rhs>
!     INTEGER, INTENT(IN) :: IPIV(:)
!     <type>(<wp>), INTENT(INOUT) :: <sol>
!     REAL(<wp>), INTENT(OUT), OPTIONAL :: <err>
!     CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
!     INTEGER, INTENT(OUT), OPTIONAL :: INFO
!     where
!     <type> ::= REAL | COMPLEX
!     <wp>   ::= KIND(1.0) | KIND(1.0D0)
!     <rhs>  ::= B(:,:) | B(:)
!     <sol>  ::= X(:,:) | X(:)
!     <err>  ::= FERR(:), BERR(:) | FERR, BERR
!
!  =====================
!
!  A     (input) either REAL or COMPLEX square array,
!        shape (:,:), size(A,1) == size(A,2).
!        The original matrix A.
!
!  AF    (input) either REAL or COMPLEX square array,
!        shape (:,:), size(AF,1) == size(AF,2) == size(A,1).
!        The factors L and U from the factorization A = PLU
!        as computed by LA_GETRF.
!
!  IPIV  (input) INTEGER array, shape (:), size(IPIV) == size(A,1).
!        The pivot indices from LA_GETRF; for 1<=i<=size(A,1), row i
!        of the matrix was interchanged with row IPIV(i).
!
!  B     (input) either REAL or COMPLEX rectangular array,
!        shape either (:,:) or (:), size(B,1) or size(B) == size(A,1).
!        The right hand side vector(s) of matrix B for
!        the system of equations AX = B.
!
!  X     (input/output) either REAL or COMPLEX rectangular array,
!        shape either (:,:) or (:), size(X,1) or size(X) == size(A,1).
!        On entry, the solution matrix X, as computed by LA_GETRS.
!        On exit, the improved solution matrix X.
!
!  TRANS Optional (input) CHARACTER*1
!        If TRANS is present, it specifies the form of the system 
!        of equations:
!           = 'N':  A X = B    (No transpose)
!           = 'T':  A^T X = B  (Transpose)
!           = 'C':  A^H X = B  (Conjugate transpose = Transpose)
!        otherwise TRANS = 'N' is assumed.
!
!  FERR  Optional (output) either REAL array of shape (:) or REAL
!        scalar. If it is an array, size(FERR) == size(X,2).
!        The estimated forward error bound for each solution vector
!        X(j) (the j-th column of the solution matrix X).
!        If XTRUE is the true solution corresponding to X(j), FERR(j)
!        is an estimated upper bound for the magnitude of the largest
!        element in (X(j) - XTRUE) divided by the magnitude of the
!        largest element in X(j).  The estimate is as reliable as
!        the estimate for RCOND, and is almost always a slight
!        overestimate of the true error.
!
!  BERR  Optional (output) either REAL array of shape (:) or REAL
!        scalar. If it is an array, size(BERR) == size(X,2).
!        The componentwise relative backward error of each solution
!        vector X(j) (i.e., the smallest relative change in
!        any element of A or B that makes X(j) an exact solution).
!
!  INFO  Optional (output) INTEGER.
!        If INFO is present
!           = 0: successful exit
!           < 0: if INFO = -k, the k-th argument had an illegal value
!        If INFO is not present and an error occurs, then the program is
!           terminated with an error message.
!
!  Internal Parameters
!  ===================
!
!  ITMAX is the maximum number of steps of iterative refinement.
!  It is set to 5 in the LAPACK77 subroutines
!
!  =====================================================================
!
      SUBROUTINE SGERFS_F90(A,AF,IPIV,B,X,TRANS,FERR,BERR,INFO)
         USE LA_PRECISION, ONLY: WP => SP
         CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(IN) :: IPIV(:)
         REAL(WP), INTENT(OUT), OPTIONAL :: FERR(:), BERR(:)
         REAL(WP), INTENT(IN) :: A(:,:), AF(:,:), B(:,:)
         REAL(WP), INTENT(INOUT) :: X(:,:)
      END SUBROUTINE SGERFS_F90
      SUBROUTINE S1GERFS_F90(A,AF,IPIV,B,X,TRANS,FERR,BERR,INFO)
         USE LA_PRECISION, ONLY: WP => SP
         CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(IN) :: IPIV(:)
         REAL(WP), INTENT(OUT), OPTIONAL :: FERR, BERR
         REAL(WP), INTENT(IN) :: A(:,:), AF(:,:), B(:)
         REAL(WP), INTENT(INOUT) :: X(:)
      END SUBROUTINE S1GERFS_F90
      SUBROUTINE DGERFS_F90(A,AF,IPIV,B,X,TRANS,FERR,BERR,INFO)
         USE LA_PRECISION, ONLY: WP => DP
         CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(IN) :: IPIV(:)
         REAL(WP), INTENT(OUT), OPTIONAL :: FERR(:), BERR(:)
         REAL(WP), INTENT(IN) :: A(:,:), AF(:,:), B(:,:)
         REAL(WP), INTENT(INOUT) :: X(:,:)
      END SUBROUTINE DGERFS_F90
      SUBROUTINE D1GERFS_F90(A,AF,IPIV,B,X,TRANS,FERR,BERR,INFO)
         USE LA_PRECISION, ONLY: WP => DP
         CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(IN) :: IPIV(:)
         REAL(WP), INTENT(OUT), OPTIONAL :: FERR, BERR
         REAL(WP), INTENT(IN) :: A(:,:), AF(:,:), B(:)
         REAL(WP), INTENT(INOUT) :: X(:)
      END SUBROUTINE D1GERFS_F90
      SUBROUTINE CGERFS_F90(A,AF,IPIV,B,X,TRANS,FERR,BERR,INFO)
         USE LA_PRECISION, ONLY: WP => SP
         CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(IN) :: IPIV(:)
         REAL(WP), INTENT(OUT), OPTIONAL :: FERR(:), BERR(:)
         COMPLEX(WP), INTENT(IN) :: A(:,:), AF(:,:), B(:,:)
         COMPLEX(WP), INTENT(INOUT) :: X(:,:)
      END SUBROUTINE CGERFS_F90
      SUBROUTINE C1GERFS_F90(A,AF,IPIV,B,X,TRANS,FERR,BERR,INFO)
         USE LA_PRECISION, ONLY: WP => SP
         CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(IN) :: IPIV(:)
         REAL(WP), INTENT(OUT), OPTIONAL :: FERR, BERR
         COMPLEX(WP), INTENT(IN) :: A(:,:), AF(:,:), B(:)
         COMPLEX(WP), INTENT(INOUT) :: X(:)
      END SUBROUTINE C1GERFS_F90
      SUBROUTINE ZGERFS_F90(A,AF,IPIV,B,X,TRANS,FERR,BERR,INFO)
         USE LA_PRECISION, ONLY: WP => DP
         CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(IN) :: IPIV(:)
         REAL(WP), INTENT(OUT), OPTIONAL :: FERR(:), BERR(:)
         COMPLEX(WP), INTENT(IN) :: A(:,:), AF(:,:), B(:,:)
         COMPLEX(WP), INTENT(INOUT) :: X(:,:)
      END SUBROUTINE ZGERFS_F90
      SUBROUTINE Z1GERFS_F90(A,AF,IPIV,B,X,TRANS,FERR,BERR,INFO)
         USE LA_PRECISION, ONLY: WP => DP
         CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         INTEGER, INTENT(IN) :: IPIV(:)
         REAL(WP), INTENT(OUT), OPTIONAL :: FERR, BERR
         COMPLEX(WP), INTENT(IN) :: A(:,:), AF(:,:), B(:)
         COMPLEX(WP), INTENT(INOUT) :: X(:)
      END SUBROUTINE Z1GERFS_F90
   END INTERFACE
!
   INTERFACE LA_GEEQU
!
!  Purpose
!  =======
!
!  LA_GEEQU computes row and column scalings intended to equilibrate a
!  rectangle matrix A and reduce its condition number.  R returns the
!  row scale factors and C the column scale factors, chosen to try to
!  make the largest entry in each row and column of the matrix B with
!  elements B(i,j) = R(i) A(i,j) C(j) have absolute value 1.
!
!  R(i) and C(j) are restricted to be between SMLNUM = smallest safe
!  number and BIGNUM = largest safe number. Use of these scaling
!  factors is not guaranteed to reduce the condition number of A but
!  works well in practice.
!
!  Arguments
!  =========
!
!  SUBROUTINE LA_GEEQU ( A, R, C, ROWCND, COLCND, AMAX, INFO )
!     <type>(<wp>), INTENT(IN) :: A(:,:)
!     REAL(<wp>), INTENT( OUT ) :: R(:), C(:)
!     REAL(<wp>), INTENT( OUT ), OPTIONAL :: ROWCND, COLCND, AMAX
!     INTEGER, INTENT(OUT), OPTIONAL :: INFO
!     where
!     <type> ::= REAL | COMPLEX
!     <wp>   ::= KIND(1.0) | KIND(1.0D0)
!
!  ====================
!
!  A       (input) either REAL or COMPLEX array, shape (:,:).
!          The matrix A, whose equilibration factors are to be computed.
!
!  R       (output) REAL array, shape (:), size(R) == size(A,1).
!          If INFO = 0 or INFO > size(A,1), R contains the row
!          scale factors for A.
!
!  C       (output) REAL array, shape (:), size(C) == size(A,2).
!          If INFO = 0, C contains the column scale factors for A.
!
!  ROWCND  Optional (output) REAL.
!          If INFO = 0 or INFO > size(A,1), ROWCND contains the ratio
!          of the smallest R(i) to the largest R(i).  If ROWCND >= 0.1
!          and AMAX is neither too large nor too small, it is not worth
!          scaling by R.
!
!  COLCND  Optional (output) REAL.
!          If INFO = 0, COLCND contains the ratio of the smallest
!          C(i) to the largest C(i).  If COLCND >= 0.1, it is not
!          worth scaling by C.
!
!  AMAX    Optional (output) REAL.
!          Absolute value of largest matrix element.  If AMAX is very
!          close to overflow or very close to underflow, the matrix
!          should be scaled.
!
!  INFO    Optional (output) INTEGER
!          If INFO is present
!             = 0:  successful exit
!             < 0:  if INFO = -k, the k-th argument had an illegal value
!             > 0:  if INFO = k,  and k is
!                   <= M:  the k-th row of A is exactly zero
!                   >  M:  the (k-M)-th column of A is exactly zero
!                          where M = size(A,1)
!          If INFO is not present and an error occurs, then the program is
!             terminated with an error message.
!
!  =====================================================================
      SUBROUTINE SGEEQU_F90( A, R, C, ROWCND, COLCND, AMAX, INFO )
         USE LA_PRECISION, ONLY: WP => SP
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         REAL(WP), INTENT( IN ) :: A( :, : )
         REAL(WP), INTENT( OUT ) :: R(:), C(:)
         REAL(WP), INTENT( OUT ), OPTIONAL :: ROWCND, COLCND, AMAX
      END SUBROUTINE SGEEQU_F90
      SUBROUTINE DGEEQU_F90( A, R, C, ROWCND, COLCND, AMAX, INFO )
         USE LA_PRECISION, ONLY: WP => DP
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         REAL(WP), INTENT( IN ) :: A( :, : )
         REAL(WP), INTENT( OUT ) :: R(:), C(:)
         REAL(WP), INTENT( OUT ), OPTIONAL :: ROWCND, COLCND, AMAX
      END SUBROUTINE DGEEQU_F90
      SUBROUTINE CGEEQU_F90( A, R, C, ROWCND, COLCND, AMAX, INFO )
         USE LA_PRECISION, ONLY: WP => SP
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         COMPLEX(WP), INTENT( IN ) :: A( :, : )
         REAL(WP), INTENT( OUT ) :: R(:), C(:)
         REAL(WP), INTENT( OUT ), OPTIONAL :: ROWCND, COLCND, AMAX
      END SUBROUTINE CGEEQU_F90
      SUBROUTINE ZGEEQU_F90( A, R, C, ROWCND, COLCND, AMAX, INFO )
         USE LA_PRECISION, ONLY: WP => DP
         INTEGER, INTENT(OUT), OPTIONAL :: INFO
         COMPLEX(WP), INTENT( IN ) :: A( :, : )
         REAL(WP), INTENT( OUT ) :: R(:), C(:)
         REAL(WP), INTENT( OUT ), OPTIONAL :: ROWCND, COLCND, AMAX
      END SUBROUTINE ZGEEQU_F90
   END INTERFACE
!
END MODULE LAPACK90_INTERFACES
