!=========================================================================!
! file pwcr_drv.f90                                                       !
!=========================================================================!
!      IMPROVED CYCLIC REDUCTION FOR SOLVING QUEUEING PROBLEMS            !
!                     by D.A. Bini and B. Meini                           !
!               (bini@dm.unipi.it    meini@dm.unipi.it)                   !
!            Fortran 90 Program version 1.0, January 30, 1997             !
!                         DRIVER PROGRAM                                  !
!=========================================================================!
! This is the driver program for solving the matrix equation              !
!                A_1+X A_2+X^2 A_3+....X^(m-1) A_m-X=O             (1)    !
! by means of pointwise cyclic reduction, where X is the (nb x nb) unknown!
! matrix and the (nb x nb) nonnegative matrices A_i, i=1,2,..., are such  !
! that  A_1+A_2+...+A_m is column stochastic.                             !
!=========================================================================!
! The program reads from the standard input the name of the file          !
! (<input_file_name>) containing the data. Then reads from the file       !
! <input_file_name> the data and calls the subroutine PWCR solving the    !
! matrix equation (1).                                                    !
! The computed  solution is stored in the file <input_file_name.out>      !
! it is written row-wise.                                                 !
! The file <input_file_name> must contain on each row the following data: !
! 1-st row: NB  ( integer ) = Block dimension                             !
! 2-nd row: M   ( integer ) = Number of blocks A_i                        !
! 3-rd row: EPS ( real(kind(0.d0)) ) = Precision level                    !
! The subsequent rows:  L, I, J, VAL,  where                              !
!    L   : ( integer ) is the index of the block A_L                      !
!    I,J : ( integer ) are the indices of the (I,J)-th entry of the block !
!           A_L                                                           !
!    VAL : ( real(kind(0.d0)) ) is the value of the (I,J)-th entry of the !
!           block A_L                                                     !
! The last row: 0,0,0,0.0d0 which denotes the end of the file.            !
! The values that are not reported in this file are intended to be zero.  !
!=========================================================================!
PROGRAM driver
  USE pwcr_interface
  IMPLICIT NONE
  REAL(KIND(0.d0)), DIMENSION(:,:,:), POINTER :: a
  REAL(KIND(0.d0)), DIMENSION(:,:), POINTER   :: g
  REAL(KIND(0.d0))                            :: err, eps, val
  INTEGER                                     :: m, nb, i,j,l
  CHARACTER(len=10)                           :: ifn
  LOGICAL                                     :: ex
  WRITE(*,*)'-------------------------------------------------------'
  WRITE(*,*)'IMPROVED CYCLIC REDUCTION FOR SOLVING QUEUEING PROBLEMS'
  WRITE(*,*)'                 by D.A. Bini and B. Meini'
  WRITE(*,*)'   Fortran 90 Program version 1.0, January 30, 1997'    
  WRITE(*,*)'-------------------------------------------------------'
  WRITE(*,*)
  WRITE(*,'(A)', advance='NO')' Input-file name = '
  READ(*,*) ifn
  INQUIRE(file=ifn, exist=ex)
  IF(.NOT.ex)THEN
     WRITE(*,*)
     WRITE(*,*)'THE FILE      ',ifn,'DOES NOT EXIST'
     STOP
  ENDIF
  OPEN(unit=10,file=ifn,status='old')
  J = INDEX(ifn,'.')
  IF (J.EQ.0) J = INDEX(ifn,' ')
  OPEN(unit=11,file=ifn(1:j-1)//'.out')
  READ(10,*) nb
  READ(10,*) m
  READ(10,*) eps
  ALLOCATE(a(nb,nb,m))
  ! reading data
  a=0.0d0
  READ(10,*) l,i,j,val
  reading : DO WHILE ( i/=0 )
     a(i,j,l)=val
     READ(10,*)l,i,j,val
  END DO reading
  CALL pwcr(a,eps,g,err)
  WRITE(11,*)'-------------------------------------------------------'
  WRITE(11,*)'IMPROVED CYCLIC REDUCTION FOR SOLVING QUEUEING PROBLEMS'
  WRITE(11,*)'                 by D.A. Bini and B. Meini'
  WRITE(11,*)'   Fortran 90 Program version 1.0, January 30, 1997'    
  WRITE(11,*)'-------------------------------------------------------'
  WRITE(11,*)
  WRITE(*,*)'Residual error=',err
  WRITE(11,*)'Residual error=',err
  WRITE(*,*)
  WRITE(11,*)
  loopw1 : DO i=1,nb
     loopw2 : DO j=1,nb
	WRITE(*,*)g(i,j)
	WRITE(11,*)g(i,j)
     END DO loopw2
  END DO loopw1
  STOP
END PROGRAM driver









