!=========================================================================!
! file pwcr_int.f90                                                       !
!=========================================================================!
!      IMPROVED CYCLIC REDUCTION FOR SOLVING QUEUEING PROBLEMS            !
!                     by D.A. Bini and B. Meini                           !
!               (bini@dm.unipi.it    meini@dm.unipi.it)                   !
!          Fortran 90 Program version 1.0, January 30 1997                !
!                 Interface File for PWCR subroutines                     !
!=========================================================================!
! Interface file for the subroutines:                                     !
!     pwcr                                                                !
!     computeG                                                            !
!     residual                                                            !
!     schur                                                               !
!     test                                                                !
!     sc1p                                                                !
!     sc2p                                                                !
!     scc2p                                                               !
!     sc1                                                                 !
!     sc2                                                                 !
!     prodc                                                               !
!     means                                                               !
!     pmeans                                                              !
!     scc2                                                                !
!     solver                                                              !
!     solvec                                                              !
!=========================================================================!   
MODULE pwcr_interface
  INTERFACE 
     SUBROUTINE pwcr(a,eps,g,err)
       IMPLICIT NONE
       REAL(KIND(0.d0)), DIMENSION(:,:,:), POINTER :: a
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER   :: g
       REAL(KIND(0.d0))                            :: err, eps
     END SUBROUTINE pwcr
  endinterface
  !=======================================================================
  INTERFACE
     SUBROUTINE computeG(a0,a0s,ae,ao,hae,hao,eps,g)
       IMPLICIT NONE
       REAL(KIND(0.d0)), DIMENSION(:,:,:), POINTER :: ae, ao, hae, hao
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER   :: a0, g
       REAL(KIND(0.d0)), DIMENSION(:), POINTER     :: a0s
       REAL(KIND(0.d0))                            :: eps
     END SUBROUTINE computeG
  endinterface
  !=======================================================================
  INTERFACE
     SUBROUTINE residual(a,g,err)
       IMPLICIT NONE
       REAL(KIND(0.d0)), DIMENSION(:,:,:), POINTER :: a
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER   :: g
       REAL(KIND(0.d0))                            :: err
     END SUBROUTINE residual
  endinterface
END MODULE pwcr_interface
!=======================================================================
!=======================================================================
MODULE schur_interface
  INTERFACE
     SUBROUTINE schur(ae,ao,hae,hao,mean,hmean,a1s,eps)
       IMPLICIT NONE
       REAL(KIND(0.d0)), DIMENSION(:,:,:), POINTER :: ae, ao, hae, hao
       REAL(KIND(0.d0)), DIMENSION(:), POINTER     :: mean, hmean, a1s
       REAL(KIND(0.d0))                            :: eps
     END SUBROUTINE schur
  END INTERFACE
  !=======================================================================
  INTERFACE 
     SUBROUTINE test(u,mean,eps,answer)
       IMPLICIT NONE
       REAL(KIND(0.d0)), POINTER, DIMENSION(:,:,:) :: u
       REAL(KIND(0.d0)), POINTER, DIMENSION(:)     :: mean
       REAL(KIND(0.d0))                            :: eps
       LOGICAL                                     :: answer
     END SUBROUTINE test
  END INTERFACE
  !=======================================================================
  INTERFACE
     SUBROUTINE sc1p(tae,tao,a)
       IMPLICIT NONE
       REAL(KIND(0.d0)), POINTER, DIMENSION(:,:,:) :: tao, tae, a
     END SUBROUTINE sc1p
  END INTERFACE
  !=======================================================================
  INTERFACE
     SUBROUTINE sc2p(tae,tao,a1,a2)
       IMPLICIT NONE
       REAL(KIND(0.d0)), POINTER, DIMENSION(:,:,:) :: tao, tae, a1, a2
     END SUBROUTINE sc2p
  END INTERFACE
  !=======================================================================
  INTERFACE
     SUBROUTINE scc2p(thae,thao,a1,a2)
       IMPLICIT NONE
       REAL(KIND(0.d0)), POINTER, DIMENSION(:,:,:) :: thao, thae, a1, a2
     END SUBROUTINE scc2p
  END INTERFACE
  !=======================================================================
  INTERFACE
     SUBROUTINE sc1(tae,tao,a)
       IMPLICIT NONE
       REAL(KIND(0.d0)), POINTER, DIMENSION(:,:,:) :: tae, tao, a
     END SUBROUTINE sc1
  END INTERFACE
  !=======================================================================
  INTERFACE
     SUBROUTINE sc2(tae,tao,a1,a2)
       IMPLICIT NONE
       REAL(KIND(0.d0)), POINTER, DIMENSION(:,:,:) :: tae, tao, a1, a2
     END SUBROUTINE sc2
  END INTERFACE
  !=======================================================================
  INTERFACE
     SUBROUTINE prodc(a,b,ir,ic,rmat,cmat)
       IMPLICIT NONE
       REAL(KIND(0.d0)), POINTER, DIMENSION(:,:,:) :: a, b
       REAL(KIND(0.d0)), POINTER, DIMENSION(:,:)   :: rmat, cmat
       INTEGER                                     :: ir,ic
     END SUBROUTINE prodc
  END INTERFACE
  !=======================================================================
  INTERFACE
     SUBROUTINE means(tae,tao,thae,mean,hmean)
       IMPLICIT NONE
       REAL(KIND(0.d0)), POINTER, DIMENSION(:,:,:) :: tae,tao, thae
       REAL(KIND(0.d0)), POINTER, DIMENSION(:)     :: mean,hmean
     END SUBROUTINE means
  END INTERFACE
  !=======================================================================
  INTERFACE
     SUBROUTINE pmeans(ae,ao,hae,hao,mean,hmean)
       IMPLICIT NONE
       REAL(KIND(0.d0)), POINTER, DIMENSION(:,:,:) :: ae,ao, hae, hao
       REAL(KIND(0.d0)), POINTER, DIMENSION(:)     :: mean,hmean
     END SUBROUTINE pmeans
  END INTERFACE
  !=======================================================================
  INTERFACE
     SUBROUTINE scc2(thae,thao,a1,a2)
       IMPLICIT NONE
       REAL(KIND(0.d0)), POINTER, DIMENSION(:,:,:) :: thae,thao, a1, a2
     END SUBROUTINE scc2
  END INTERFACE
  !=======================================================================
  INTERFACE
     SUBROUTINE solver(rmat,termr)
       IMPLICIT NONE
       REAL(KIND(0.d0)), POINTER, DIMENSION(:,:) :: rmat,termr
     END SUBROUTINE solver
  END INTERFACE
  !=======================================================================
  INTERFACE
     SUBROUTINE solvec(cmat,termc)    
       IMPLICIT NONE
       COMPLEX(KIND(0.d0)), POINTER, DIMENSION(:,:) :: cmat,termc
     END SUBROUTINE solvec
  END INTERFACE

END MODULE schur_interface














